@@ -57,6 +57,7 @@
 #include "util-print.h"
 #include "util-proto-name.h"
 #include "util-optimize.h"
+#include "util-logopenfile.h"
 
 #define DEFAULT_LOG_FILENAME "fast.log"
 
@@ -68,7 +69,6 @@ TmEcode AlertFastLogIPv6(ThreadVars *, Packet *, void *, PacketQueue *, PacketQu
 TmEcode AlertFastLogThreadInit(ThreadVars *, void *, void **);
 TmEcode AlertFastLogThreadDeinit(ThreadVars *, void *);
 void AlertFastLogExitPrintStats(ThreadVars *, void *);
-static int AlertFastLogOpenFileCtx(LogFileCtx *, const char *, const char *);
 void AlertFastLogRegisterTests(void);
 static void AlertFastLogDeInitCtx(OutputCtx *);
 
@@ -145,9 +145,9 @@ TmEcode AlertFastLogIPv4(ThreadVars *tv, Packet *p, void *data, PacketQueue *pq,
         PrintInet(AF_INET, (const void *)GET_IPV4_SRC_ADDR_PTR(p), srcip, sizeof(srcip));
         PrintInet(AF_INET, (const void *)GET_IPV4_DST_ADDR_PTR(p), dstip, sizeof(dstip));
 
-        if (pa->action == ACTION_DROP && IS_ENGINE_MODE_IPS(engine_mode)) {
+        if (pa->action & ACTION_DROP && IS_ENGINE_MODE_IPS(engine_mode)) {
             action = "[Drop] ";
-        } else if (pa->action == ACTION_DROP) {
+        } else if (pa->action & ACTION_DROP) {
             action = "[wDrop] ";
         }
 
@@ -199,9 +199,9 @@ TmEcode AlertFastLogIPv6(ThreadVars *tv, Packet *p, void *data, PacketQueue *pq,
         PrintInet(AF_INET6, (const void *)GET_IPV6_SRC_ADDR(p), srcip, sizeof(srcip));
         PrintInet(AF_INET6, (const void *)GET_IPV6_DST_ADDR(p), dstip, sizeof(dstip));
 
-        if (pa->action == ACTION_DROP && IS_ENGINE_MODE_IPS(engine_mode)) {
+        if (pa->action & ACTION_DROP && IS_ENGINE_MODE_IPS(engine_mode)) {
             action = "[Drop] ";
-        } else if (pa->action == ACTION_DROP) {
+        } else if (pa->action & ACTION_DROP) {
             action = "[wDrop] ";
         }
 
@@ -251,9 +251,9 @@ TmEcode AlertFastLogDecoderEvent(ThreadVars *tv, Packet *p, void *data, PacketQu
             continue;
         }
 
-        if (pa->action == ACTION_DROP && IS_ENGINE_MODE_IPS(engine_mode)) {
+        if (pa->action & ACTION_DROP && IS_ENGINE_MODE_IPS(engine_mode)) {
             action = "[Drop] ";
-        } else if (pa->action == ACTION_DROP) {
+        } else if (pa->action & ACTION_DROP) {
             action = "[wDrop] ";
         }
 
@@ -328,7 +328,7 @@ void AlertFastLogExitPrintStats(ThreadVars *tv, void *data) {
         return;
     }
 
-    SCLogInfo("(%s) Alerts %" PRIu64 "", tv->name, aft->file_ctx->alerts);
+    SCLogInfo("Fast log output wrote %" PRIu64 " alerts", aft->file_ctx->alerts);
 }
 
 /**
@@ -344,15 +344,7 @@ OutputCtx *AlertFastLogInitCtx(ConfNode *conf)
         return NULL;
     }
 
-    const char *filename = ConfNodeLookupChildValue(conf, "filename");
-    if (filename == NULL)
-        filename = DEFAULT_LOG_FILENAME;
-
-    const char *mode = ConfNodeLookupChildValue(conf, "append");
-    if (mode == NULL)
-        mode = DEFAULT_LOG_MODE_APPEND;
-
-    if (AlertFastLogOpenFileCtx(logfile_ctx, filename, mode) < 0) {
+    if (SCConfLogOpenGeneric(conf, logfile_ctx, DEFAULT_LOG_FILENAME) < 0) {
         LogFileFreeCtx(logfile_ctx);
         return NULL;
     }
@@ -363,8 +355,6 @@ OutputCtx *AlertFastLogInitCtx(ConfNode *conf)
     output_ctx->data = logfile_ctx;
     output_ctx->DeInit = AlertFastLogDeInitCtx;
 
-    SCLogInfo("Fast log output initialized, filename: %s", filename);
-
     return output_ctx;
 }
 
@@ -375,39 +365,6 @@ static void AlertFastLogDeInitCtx(OutputCtx *output_ctx)
     SCFree(output_ctx);
 }
 
-/** \brief Read the config set the file pointer, open the file
- *  \param file_ctx pointer to a created LogFileCtx using LogFileNewCtx()
- *  \param filename name of log file
- *  \param mode append mode (bool)
- *  \return -1 if failure, 0 if succesful
- * */
-static int AlertFastLogOpenFileCtx(LogFileCtx *file_ctx, const char *filename,
-                                    const char *mode)
-{
-    char log_path[PATH_MAX];
-    char *log_dir;
-
-    if (ConfGet("default-log-dir", &log_dir) != 1)
-        log_dir = DEFAULT_LOG_DIR;
-
-    snprintf(log_path, PATH_MAX, "%s/%s", log_dir, filename);
-
-    if (ConfValIsTrue(mode)) {
-        file_ctx->fp = fopen(log_path, "a");
-    } else {
-        file_ctx->fp = fopen(log_path, "w");
-    }
-
-    if (file_ctx->fp == NULL) {
-        SCLogError(SC_ERR_FOPEN, "failed to open %s: %s", log_path,
-                strerror(errno));
-        return -1;
-    }
-
-    return 0;
-}
-
-
 /*------------------------------Unittests-------------------------------------*/
 
 #ifdef UNITTESTS
