@@ -25,7 +25,7 @@
 #define __CONF_YAML_LOADER_H__
 
 int ConfYamlLoadFile(const char *);
-void ConfYamlLoadString(const char *, size_t);
+int ConfYamlLoadString(const char *, size_t);
 void ConfYamlRegisterTests(void);
 
 #endif /* !__CONF_YAML_LOADER_H__ */
