@@ -25,6 +25,7 @@
 #define __DECODE_ICMPV6_H__
 
 #include "decode-tcp.h"
+#include "decode-sctp.h"
 #include "decode-udp.h"
 #include "decode-ipv6.h"
 
@@ -78,10 +79,8 @@
 
 /** If message is informational */
 /** macro for icmpv6 "id" access */
-/* #define ICMPV6_GET_ID(p)        (p)->icmpv6h->icmpv6b.icmpv6i.id */
-#define ICMPV6_GET_ID(p)        (p)->icmpv6vars.id
+#define ICMPV6_GET_ID(p)        (ntohs((p)->icmpv6vars.id))
 /** macro for icmpv6 "seq" access */
-/* #define ICMPV6_GET_SEQ(p)       (p)->icmpv6h->icmpv6b.icmpv6i.seq */
 #define ICMPV6_GET_SEQ(p)       (ntohs((p)->icmpv6vars.seq))
 
 /** If message is Error */
@@ -116,7 +115,7 @@ typedef struct ICMPV6Hdr_
     uint8_t  code;
     uint16_t csum;
 
-    union{
+    union {
         ICMPV6Info icmpv6i; /** Informational message */
         union
         {
@@ -155,6 +154,7 @@ typedef struct ICMPV6Vars_ {
 
 } ICMPV6Vars;
 
+
 #define CLEAR_ICMPV6_PACKET(p) do { \
     (p)->icmpv6vars.comp_csum = -1; \
     (p)->icmpv6vars.id = 0; \
