@@ -15,6 +15,13 @@
  * 02110-1301, USA.
  */
 
+/**
+ * \ingroup decode
+ *
+ * @{
+ */
+
+
 /**
  * \file
  *
@@ -35,19 +42,19 @@
 static int DecodeUDPPacket(ThreadVars *t, Packet *p, uint8_t *pkt, uint16_t len)
 {
     if (len < UDP_HEADER_LEN) {
-        DECODER_SET_EVENT(p, UDP_HLEN_TOO_SMALL);
+        ENGINE_SET_EVENT(p, UDP_HLEN_TOO_SMALL);
         return -1;
     }
 
     p->udph = (UDPHdr *)pkt;
 
     if (len < UDP_GET_LEN(p)) {
-        DECODER_SET_EVENT(p, UDP_PKT_TOO_SMALL);
+        ENGINE_SET_EVENT(p, UDP_PKT_TOO_SMALL);
         return -1;
     }
 
     if (len != UDP_GET_LEN(p)) {
-        DECODER_SET_EVENT(p, UDP_HLEN_INVALID);
+        ENGINE_SET_EVENT(p, UDP_HLEN_INVALID);
         return -1;
     }
 
@@ -198,3 +205,6 @@ void DecodeUDPV4RegisterTests(void)
                    UDPV6CalculateInvalidChecksumtest04, 0);
 #endif /* UNITTESTS */
 }
+/**
+ * @}
+ */
