@@ -19,7 +19,7 @@
  * \file
  *
  * \author Victor Julien <victor@inliniac.net>
- * \author Anoop Saldanha <poonaatsoc@gmail.com>
+ * \author Anoop Saldanha <anoopsaldanha@gmail.com>
  *
  * Implements the distance keyword
  */
@@ -86,19 +86,19 @@ static int DetectDistanceSetup (DetectEngineCtx *de_ctx, Signature *s,
         SigMatch *pm2_ots = NULL;
 
         dcem = SigMatchGetLastSMFromLists(s, 6,
-                                          DETECT_DCE_IFACE, s->sm_lists_tail[DETECT_SM_LIST_AMATCH],
-                                          DETECT_DCE_OPNUM, s->sm_lists_tail[DETECT_SM_LIST_AMATCH],
-                                          DETECT_DCE_STUB_DATA, s->sm_lists_tail[DETECT_SM_LIST_AMATCH]);
+                DETECT_DCE_IFACE, s->sm_lists_tail[DETECT_SM_LIST_AMATCH],
+                DETECT_DCE_OPNUM, s->sm_lists_tail[DETECT_SM_LIST_AMATCH],
+                DETECT_DCE_STUB_DATA, s->sm_lists_tail[DETECT_SM_LIST_AMATCH]);
 
         pm1_ots = SigMatchGetLastSMFromLists(s, 6,
-                                             DETECT_CONTENT, s->sm_lists_tail[DETECT_SM_LIST_PMATCH],
-                                             DETECT_PCRE, s->sm_lists_tail[DETECT_SM_LIST_PMATCH],
-                                             DETECT_BYTEJUMP, s->sm_lists_tail[DETECT_SM_LIST_PMATCH]);
+                DETECT_CONTENT, s->sm_lists_tail[DETECT_SM_LIST_PMATCH],
+                DETECT_PCRE, s->sm_lists_tail[DETECT_SM_LIST_PMATCH],
+                DETECT_BYTEJUMP, s->sm_lists_tail[DETECT_SM_LIST_PMATCH]);
         if (pm1_ots != NULL && pm1_ots->prev != NULL) {
             pm2_ots = SigMatchGetLastSMFromLists(s, 6,
-                                                 DETECT_CONTENT, pm1_ots->prev,
-                                                 DETECT_PCRE, pm1_ots->prev,
-                                                 DETECT_BYTEJUMP, pm1_ots->prev);
+                    DETECT_CONTENT, pm1_ots->prev,
+                    DETECT_PCRE, pm1_ots->prev,
+                    DETECT_BYTEJUMP, pm1_ots->prev);
         }
 
         dm = SigMatchGetLastSMFromLists(s, 2, DETECT_CONTENT, s->sm_lists_tail[DETECT_SM_LIST_DMATCH]);
@@ -115,10 +115,10 @@ static int DetectDistanceSetup (DetectEngineCtx *de_ctx, Signature *s,
                 if (pm1->idx > dcem->idx) {
                     /* transfer pm1 to dmatch list and within is against this */
                     SigMatchTransferSigMatchAcrossLists(pm1,
-                                                        &s->sm_lists[DETECT_SM_LIST_PMATCH],
-                                                        &s->sm_lists_tail[DETECT_SM_LIST_PMATCH],
-                                                        &s->sm_lists[DETECT_SM_LIST_DMATCH],
-                                                        &s->sm_lists_tail[DETECT_SM_LIST_DMATCH]);
+                            &s->sm_lists[DETECT_SM_LIST_PMATCH],
+                            &s->sm_lists_tail[DETECT_SM_LIST_PMATCH],
+                            &s->sm_lists[DETECT_SM_LIST_DMATCH],
+                            &s->sm_lists_tail[DETECT_SM_LIST_DMATCH]);
                     pm = pm1;
                 } else {
                     /* within is against pm1 and we continue this way */
@@ -130,10 +130,10 @@ static int DetectDistanceSetup (DetectEngineCtx *de_ctx, Signature *s,
             } else if (pm1->idx > dcem->idx) {
                 /* transfer pm1 to dmatch list and within is against this */
                 SigMatchTransferSigMatchAcrossLists(pm1,
-                                                    &s->sm_lists[DETECT_SM_LIST_PMATCH],
-                                                    &s->sm_lists_tail[DETECT_SM_LIST_PMATCH],
-                                                    &s->sm_lists[DETECT_SM_LIST_DMATCH],
-                                                    &s->sm_lists_tail[DETECT_SM_LIST_DMATCH]);
+                        &s->sm_lists[DETECT_SM_LIST_PMATCH],
+                        &s->sm_lists_tail[DETECT_SM_LIST_PMATCH],
+                        &s->sm_lists[DETECT_SM_LIST_DMATCH],
+                        &s->sm_lists_tail[DETECT_SM_LIST_DMATCH]);
                 pm = pm1;
             } else {
                 /* within is against pm1 and we continue this way */
@@ -149,10 +149,10 @@ static int DetectDistanceSetup (DetectEngineCtx *de_ctx, Signature *s,
             } else if (pm2_ots == NULL || pm2_ots->idx < dcem->idx) {
                 /* trasnfer pm1 to dmatch list and pm = pm1 */
                 SigMatchTransferSigMatchAcrossLists(pm1,
-                                                    &s->sm_lists[DETECT_SM_LIST_PMATCH],
-                                                    &s->sm_lists_tail[DETECT_SM_LIST_PMATCH],
-                                                    &s->sm_lists[DETECT_SM_LIST_DMATCH],
-                                                    &s->sm_lists_tail[DETECT_SM_LIST_DMATCH]);
+                        &s->sm_lists[DETECT_SM_LIST_PMATCH],
+                        &s->sm_lists_tail[DETECT_SM_LIST_PMATCH],
+                        &s->sm_lists[DETECT_SM_LIST_DMATCH],
+                        &s->sm_lists_tail[DETECT_SM_LIST_DMATCH]);
                 pm = pm1;
             } else {
                 /* within is against pm1, pm = pm1 */
@@ -160,19 +160,20 @@ static int DetectDistanceSetup (DetectEngineCtx *de_ctx, Signature *s,
             }
         }
     } else {
-        pm = SigMatchGetLastSMFromLists(s, 16,
-                                        DETECT_CONTENT, s->sm_lists_tail[DETECT_SM_LIST_PMATCH],
-                                        DETECT_URICONTENT, s->sm_lists_tail[DETECT_SM_LIST_UMATCH],
-                                        DETECT_AL_HTTP_RAW_URI, s->sm_lists_tail[DETECT_SM_LIST_HRUDMATCH],
-                                        DETECT_AL_HTTP_CLIENT_BODY, s->sm_lists_tail[DETECT_SM_LIST_HCBDMATCH],
-                                        DETECT_AL_HTTP_HEADER, s->sm_lists_tail[DETECT_SM_LIST_HHDMATCH],
-                                        DETECT_AL_HTTP_RAW_HEADER, s->sm_lists_tail[DETECT_SM_LIST_HRHDMATCH],
-                                        DETECT_AL_HTTP_METHOD, s->sm_lists_tail[DETECT_SM_LIST_HMDMATCH],
-                                        DETECT_AL_HTTP_COOKIE, s->sm_lists_tail[DETECT_SM_LIST_HCDMATCH]);
+        pm = SigMatchGetLastSMFromLists(s, 18,
+                DETECT_CONTENT, s->sm_lists_tail[DETECT_SM_LIST_PMATCH],
+                DETECT_URICONTENT, s->sm_lists_tail[DETECT_SM_LIST_UMATCH],
+                DETECT_AL_HTTP_RAW_URI, s->sm_lists_tail[DETECT_SM_LIST_HRUDMATCH],
+                DETECT_AL_HTTP_CLIENT_BODY, s->sm_lists_tail[DETECT_SM_LIST_HCBDMATCH],
+                DETECT_AL_HTTP_SERVER_BODY, s->sm_lists_tail[DETECT_SM_LIST_HSBDMATCH],
+                DETECT_AL_HTTP_HEADER, s->sm_lists_tail[DETECT_SM_LIST_HHDMATCH],
+                DETECT_AL_HTTP_RAW_HEADER, s->sm_lists_tail[DETECT_SM_LIST_HRHDMATCH],
+                DETECT_AL_HTTP_METHOD, s->sm_lists_tail[DETECT_SM_LIST_HMDMATCH],
+                DETECT_AL_HTTP_COOKIE, s->sm_lists_tail[DETECT_SM_LIST_HCDMATCH]);
         if (pm == NULL) {
             SCLogError(SC_ERR_WITHIN_MISSING_CONTENT, "within needs"
                        "preceeding content, uricontent option, http_client_body, "
-                       "http_header, http_raw_header, http_method, "
+                       "http_server_body, http_header, http_raw_header, http_method, "
                        "http_cookie or http_raw_uri option");
             if (dubbed)
                 SCFree(str);
@@ -461,6 +462,87 @@ static int DetectDistanceSetup (DetectEngineCtx *de_ctx, Signature *s,
 
             break;
 
+        case DETECT_AL_HTTP_SERVER_BODY:
+            cd = (DetectContentData *)pm->ctx;
+
+            if (str[0] != '-' && isalpha(str[0])) {
+                SigMatch *bed_sm =
+                    DetectByteExtractRetrieveSMVar(str, s,
+                                                   SigMatchListSMBelongsTo(s, pm));
+                if (bed_sm == NULL) {
+                    SCLogError(SC_ERR_INVALID_SIGNATURE, "Unknown byte_extract var "
+                               "seen in distance - %s\n", str);
+                    goto error;
+                }
+                cd->distance = ((DetectByteExtractData *)bed_sm->ctx)->local_id;
+                cd->flags |= DETECT_CONTENT_DISTANCE_BE;
+            } else {
+                cd->distance = strtol(str, NULL, 10);
+                if (cd->flags & DETECT_CONTENT_WITHIN) {
+                    if ((cd->distance + cd->content_len) > cd->within) {
+                        cd->within = cd->distance + cd->content_len;
+                    }
+                }
+            }
+
+            if (cd->flags & DETECT_CONTENT_NEGATED) {
+                if (cd->flags & DETECT_CONTENT_FAST_PATTERN) {
+                    SCLogError(SC_ERR_INVALID_SIGNATURE, "You can't have a relative "
+                               "negated keyword set along with a fast_pattern");
+                    goto error;
+                }
+            } else {
+                if (cd->flags & DETECT_CONTENT_FAST_PATTERN_ONLY) {
+                    SCLogError(SC_ERR_INVALID_SIGNATURE, "You can't have a relative "
+                               "keyword set along with a fast_pattern:only;");
+                    goto error;
+                }
+            }
+
+            cd->flags |= DETECT_CONTENT_DISTANCE;
+
+            /* reassigning pm */
+            pm = SigMatchGetLastSMFromLists(s, 4,
+                                            DETECT_AL_HTTP_SERVER_BODY, pm->prev,
+                                            DETECT_PCRE, pm->prev);
+            if (pm == NULL) {
+                if (s->init_flags & SIG_FLAG_INIT_FILE_DATA) {
+                    /* file_data; content:"abc"; distance:0; is valid, in this case
+                     * there will be no previous pm. We convert to offset in this case */
+                    cd->flags &= ~DETECT_CONTENT_DISTANCE;
+
+                    if (cd->distance < 0) {
+                        SCLogError(SC_ERR_INVALID_SIGNATURE, "can't have negative distance relative to file_data");
+                        goto error;
+                    }
+
+                    SCLogDebug("converted distance to offset for content relative to file_data");
+                    cd->offset = cd->distance;
+                } else {
+                    SCLogError(SC_ERR_DISTANCE_MISSING_CONTENT, "distance for http_server_body "
+                            "needs preceeding http_server_body content");
+                    goto error;
+                }
+            } else {
+                if (pm->type == DETECT_PCRE) {
+                    DetectPcreData *tmp_pd = (DetectPcreData *)pm->ctx;
+                    tmp_pd->flags |=  DETECT_PCRE_RELATIVE_NEXT;
+                } else {
+                    /* reassigning cd */
+                    cd = (DetectContentData *)pm->ctx;
+                    if (cd->flags & DETECT_CONTENT_FAST_PATTERN_ONLY) {
+                        SCLogError(SC_ERR_INVALID_SIGNATURE, "Previous keyword "
+                                "has a fast_pattern:only; set.  You can't "
+                                "have relative keywords around a fast_pattern "
+                                "only content");
+                        goto error;
+                    }
+                    cd->flags |= DETECT_CONTENT_RELATIVE_NEXT;
+                }
+            }
+
+            break;
+
         case DETECT_AL_HTTP_HEADER:
             cd = (DetectContentData *)pm->ctx;
             if (cd->flags & DETECT_CONTENT_NEGATED) {
