@@ -42,7 +42,7 @@
  *  Regex (by Brian Rectanus)
  *  flags: [!+*](SAPRFU120)[,SAPRFU12]
  */
-#define PARSE_REGEX "^\\s*(?:([\\+\\*!]))?\\s*([SAPRFU120\\+\\*!]+)(?:\\s*,\\s*([SAPRFU12]+))?\\s*$"
+#define PARSE_REGEX "^\\s*(?:([\\+\\*!]))?\\s*([SAPRFU120CE\\+\\*!]+)(?:\\s*,\\s*([SAPRFU12CE]+))?\\s*$"
 
 /**
  * Flags args[0] *(3) +(2) !(1)
@@ -114,7 +114,7 @@ static int DetectFlagsMatch (ThreadVars *t, DetectEngineThreadCtx *det_ctx, Pack
     uint8_t flags = 0;
     DetectFlagsData *de = (DetectFlagsData *)m->ctx;
 
-    if(!(PKT_IS_TCP(p))) {
+    if (!(PKT_IS_TCP(p)) || PKT_IS_PSEUDOPKT(p)) {
         SCReturnInt(0);
     }
 
@@ -259,6 +259,16 @@ static DetectFlagsData *DetectFlagsParse (char *rawstr)
                     de->flags |= TH_ECN;
                     found++;
                     break;
+                case 'C':
+                case 'c':
+                    de->flags |= TH_CWR;
+                    found++;
+                    break;
+                case 'E':
+                case 'e':
+                    de->flags |= TH_ECN;
+                    found++;
+                    break;
                 case '0':
                     de->flags = 0;
                     found++;
@@ -323,6 +333,16 @@ static DetectFlagsData *DetectFlagsParse (char *rawstr)
                 de->flags |= TH_ECN;
                 found++;
                 break;
+            case 'C':
+            case 'c':
+                de->flags |= TH_CWR;
+                found++;
+                break;
+            case 'E':
+            case 'e':
+                de->flags |= TH_ECN;
+                found++;
+                break;
             case '0':
                 de->flags = 0;
                 found++;
@@ -410,6 +430,16 @@ static DetectFlagsData *DetectFlagsParse (char *rawstr)
                     de->ignored_flags &= ~TH_ECN;
                     ignore++;
                     break;
+                case 'C':
+                case 'c':
+                    de->ignored_flags &= ~TH_CWR;
+                    ignore++;
+                    break;
+                case 'E':
+                case 'e':
+                    de->ignored_flags &= ~TH_ECN;
+                    ignore++;
+                    break;
                 case '0':
                     break;
                 default:
@@ -469,6 +499,8 @@ static int DetectFlagsSetup (DetectEngineCtx *de_ctx, Signature *s, char *rawstr
     sm->ctx = (void *)de;
 
     SigMatchAppendPacket(s, sm);
+    s->flags |= SIG_FLAG_REQUIRE_PACKET;
+
     return 0;
 
 error:
@@ -534,7 +566,9 @@ static int FlagsTestParse02 (void) {
  *  \retval 0 on failure
  */
 static int FlagsTestParse03 (void) {
-    Packet p;
+    Packet *p = SCMalloc(SIZE_OF_PACKET);
+    if (p == NULL)
+        return 0;
     ThreadVars tv;
     int ret = 0;
     DetectFlagsData *de = NULL;
@@ -543,13 +577,14 @@ static int FlagsTestParse03 (void) {
     TCPHdr tcph;
 
     memset(&tv, 0, sizeof(ThreadVars));
-    memset(&p, 0, sizeof(Packet));
+    memset(p, 0, SIZE_OF_PACKET);
+    p->pkt = (uint8_t *)(p + 1);
     memset(&ipv4h, 0, sizeof(IPV4Hdr));
     memset(&tcph, 0, sizeof(TCPHdr));
 
-    p.ip4h = &ipv4h;
-    p.tcph = &tcph;
-    p.tcph->th_flags = TH_ACK|TH_PUSH|TH_SYN|TH_RST;
+    p->ip4h = &ipv4h;
+    p->tcph = &tcph;
+    p->tcph->th_flags = TH_ACK|TH_PUSH|TH_SYN|TH_RST;
 
     de = DetectFlagsParse("AP+");
 
@@ -563,17 +598,19 @@ static int FlagsTestParse03 (void) {
     sm->type = DETECT_FLAGS;
     sm->ctx = (void *)de;
 
-    ret = DetectFlagsMatch(&tv,NULL,&p,NULL,sm);
+    ret = DetectFlagsMatch(&tv,NULL,p,NULL,sm);
 
     if(ret) {
         if (de) SCFree(de);
         if (sm) SCFree(sm);
+        SCFree(p);
         return 1;
     }
 
 error:
     if (de) SCFree(de);
     if (sm) SCFree(sm);
+    SCFree(p);
     return 0;
 }
 
@@ -584,7 +621,9 @@ error:
  *  \retval 0 on failure
  */
 static int FlagsTestParse04 (void) {
-    Packet p;
+    Packet *p = SCMalloc(SIZE_OF_PACKET);
+    if (p == NULL)
+        return 0;
     ThreadVars tv;
     int ret = 0;
     DetectFlagsData *de = NULL;
@@ -593,13 +632,14 @@ static int FlagsTestParse04 (void) {
     TCPHdr tcph;
 
     memset(&tv, 0, sizeof(ThreadVars));
-    memset(&p, 0, sizeof(Packet));
+    memset(p, 0, SIZE_OF_PACKET);
+    p->pkt = (uint8_t *)(p + 1);
     memset(&ipv4h, 0, sizeof(IPV4Hdr));
     memset(&tcph, 0, sizeof(TCPHdr));
 
-    p.ip4h = &ipv4h;
-    p.tcph = &tcph;
-    p.tcph->th_flags = TH_SYN;
+    p->ip4h = &ipv4h;
+    p->tcph = &tcph;
+    p->tcph->th_flags = TH_SYN;
 
     de = DetectFlagsParse("A");
 
@@ -613,17 +653,19 @@ static int FlagsTestParse04 (void) {
     sm->type = DETECT_FLAGS;
     sm->ctx = (void *)de;
 
-    ret = DetectFlagsMatch(&tv,NULL,&p,NULL,sm);
+    ret = DetectFlagsMatch(&tv,NULL,p,NULL,sm);
 
     if(ret) {
         if (de) SCFree(de);
         if (sm) SCFree(sm);
+        SCFree(p);
         return 1;
     }
 
 error:
     if (de) SCFree(de);
     if (sm) SCFree(sm);
+    SCFree(p);
     return 0;
 }
 
@@ -634,7 +676,9 @@ error:
  *  \retval 0 on failure
  */
 static int FlagsTestParse05 (void) {
-    Packet p;
+    Packet *p = SCMalloc(SIZE_OF_PACKET);
+    if (p == NULL)
+        return 0;
     ThreadVars tv;
     int ret = 0;
     DetectFlagsData *de = NULL;
@@ -643,13 +687,14 @@ static int FlagsTestParse05 (void) {
     TCPHdr tcph;
 
     memset(&tv, 0, sizeof(ThreadVars));
-    memset(&p, 0, sizeof(Packet));
+    memset(p, 0, SIZE_OF_PACKET);
+    p->pkt = (uint8_t *)(p + 1);
     memset(&ipv4h, 0, sizeof(IPV4Hdr));
     memset(&tcph, 0, sizeof(TCPHdr));
 
-    p.ip4h = &ipv4h;
-    p.tcph = &tcph;
-    p.tcph->th_flags = TH_ACK|TH_PUSH|TH_SYN|TH_RST;
+    p->ip4h = &ipv4h;
+    p->tcph = &tcph;
+    p->tcph->th_flags = TH_ACK|TH_PUSH|TH_SYN|TH_RST;
 
     de = DetectFlagsParse("+AP,SR");
 
@@ -663,17 +708,19 @@ static int FlagsTestParse05 (void) {
     sm->type = DETECT_FLAGS;
     sm->ctx = (void *)de;
 
-    ret = DetectFlagsMatch(&tv,NULL,&p,NULL,sm);
+    ret = DetectFlagsMatch(&tv,NULL,p,NULL,sm);
 
     if(ret) {
         if (de) SCFree(de);
         if (sm) SCFree(sm);
+        SCFree(p);
         return 1;
     }
 
 error:
     if (de) SCFree(de);
     if (sm) SCFree(sm);
+    SCFree(p);
     return 0;
 }
 
@@ -684,7 +731,9 @@ error:
  *  \retval 0 on failure
  */
 static int FlagsTestParse06 (void) {
-    Packet p;
+    Packet *p = SCMalloc(SIZE_OF_PACKET);
+    if (p == NULL)
+        return 0;
     ThreadVars tv;
     int ret = 0;
     DetectFlagsData *de = NULL;
@@ -693,13 +742,14 @@ static int FlagsTestParse06 (void) {
     TCPHdr tcph;
 
     memset(&tv, 0, sizeof(ThreadVars));
-    memset(&p, 0, sizeof(Packet));
+    memset(p, 0, SIZE_OF_PACKET);
+    p->pkt = (uint8_t *)(p + 1);
     memset(&ipv4h, 0, sizeof(IPV4Hdr));
     memset(&tcph, 0, sizeof(TCPHdr));
 
-    p.ip4h = &ipv4h;
-    p.tcph = &tcph;
-    p.tcph->th_flags = TH_ACK|TH_PUSH|TH_SYN|TH_RST;
+    p->ip4h = &ipv4h;
+    p->tcph = &tcph;
+    p->tcph->th_flags = TH_ACK|TH_PUSH|TH_SYN|TH_RST;
 
     de = DetectFlagsParse("+AP,UR");
 
@@ -713,17 +763,19 @@ static int FlagsTestParse06 (void) {
     sm->type = DETECT_FLAGS;
     sm->ctx = (void *)de;
 
-    ret = DetectFlagsMatch(&tv,NULL,&p,NULL,sm);
+    ret = DetectFlagsMatch(&tv,NULL,p,NULL,sm);
 
     if(ret) {
         if (de) SCFree(de);
         if (sm) SCFree(sm);
+        SCFree(p);
         return 1;
     }
 
 error:
     if (de) SCFree(de);
     if (sm) SCFree(sm);
+    SCFree(p);
     return 0;
 }
 
@@ -734,7 +786,9 @@ error:
  *  \retval 0 on failure
  */
 static int FlagsTestParse07 (void) {
-    Packet p;
+    Packet *p = SCMalloc(SIZE_OF_PACKET);
+    if (p == NULL)
+        return 0;
     ThreadVars tv;
     int ret = 0;
     DetectFlagsData *de = NULL;
@@ -743,13 +797,14 @@ static int FlagsTestParse07 (void) {
     TCPHdr tcph;
 
     memset(&tv, 0, sizeof(ThreadVars));
-    memset(&p, 0, sizeof(Packet));
+    memset(p, 0, SIZE_OF_PACKET);
+    p->pkt = (uint8_t *)(p + 1);
     memset(&ipv4h, 0, sizeof(IPV4Hdr));
     memset(&tcph, 0, sizeof(TCPHdr));
 
-    p.ip4h = &ipv4h;
-    p.tcph = &tcph;
-    p.tcph->th_flags = TH_SYN|TH_RST;
+    p->ip4h = &ipv4h;
+    p->tcph = &tcph;
+    p->tcph->th_flags = TH_SYN|TH_RST;
 
     de = DetectFlagsParse("*AP");
 
@@ -763,17 +818,19 @@ static int FlagsTestParse07 (void) {
     sm->type = DETECT_FLAGS;
     sm->ctx = (void *)de;
 
-    ret = DetectFlagsMatch(&tv,NULL,&p,NULL,sm);
+    ret = DetectFlagsMatch(&tv,NULL,p,NULL,sm);
 
     if(ret) {
         if (de) SCFree(de);
         if (sm) SCFree(sm);
+        SCFree(p);
         return 1;
     }
 
 error:
     if (de) SCFree(de);
     if (sm) SCFree(sm);
+    SCFree(p);
     return 0;
 }
 
@@ -784,7 +841,9 @@ error:
  *  \retval 0 on failure
  */
 static int FlagsTestParse08 (void) {
-    Packet p;
+    Packet *p = SCMalloc(SIZE_OF_PACKET);
+    if (p == NULL)
+        return 0;
     ThreadVars tv;
     int ret = 0;
     DetectFlagsData *de = NULL;
@@ -793,13 +852,14 @@ static int FlagsTestParse08 (void) {
     TCPHdr tcph;
 
     memset(&tv, 0, sizeof(ThreadVars));
-    memset(&p, 0, sizeof(Packet));
+    memset(p, 0, SIZE_OF_PACKET);
+    p->pkt = (uint8_t *)(p + 1);
     memset(&ipv4h, 0, sizeof(IPV4Hdr));
     memset(&tcph, 0, sizeof(TCPHdr));
 
-    p.ip4h = &ipv4h;
-    p.tcph = &tcph;
-    p.tcph->th_flags = TH_SYN|TH_RST;
+    p->ip4h = &ipv4h;
+    p->tcph = &tcph;
+    p->tcph->th_flags = TH_SYN|TH_RST;
 
     de = DetectFlagsParse("*SA");
 
@@ -813,17 +873,19 @@ static int FlagsTestParse08 (void) {
     sm->type = DETECT_FLAGS;
     sm->ctx = (void *)de;
 
-    ret = DetectFlagsMatch(&tv,NULL,&p,NULL,sm);
+    ret = DetectFlagsMatch(&tv,NULL,p,NULL,sm);
 
     if(ret) {
         if (de) SCFree(de);
         if (sm) SCFree(sm);
+        SCFree(p);
         return 1;
     }
 
 error:
     if (de) SCFree(de);
     if (sm) SCFree(sm);
+    SCFree(p);
     return 0;
 }
 
@@ -834,7 +896,9 @@ error:
  *  \retval 0 on failure
  */
 static int FlagsTestParse09 (void) {
-    Packet p;
+    Packet *p = SCMalloc(SIZE_OF_PACKET);
+    if (p == NULL)
+        return 0;
     ThreadVars tv;
     int ret = 0;
     DetectFlagsData *de = NULL;
@@ -843,13 +907,14 @@ static int FlagsTestParse09 (void) {
     TCPHdr tcph;
 
     memset(&tv, 0, sizeof(ThreadVars));
-    memset(&p, 0, sizeof(Packet));
+    memset(p, 0, SIZE_OF_PACKET);
+    p->pkt = (uint8_t *)(p + 1);
     memset(&ipv4h, 0, sizeof(IPV4Hdr));
     memset(&tcph, 0, sizeof(TCPHdr));
 
-    p.ip4h = &ipv4h;
-    p.tcph = &tcph;
-    p.tcph->th_flags = TH_SYN|TH_RST;
+    p->ip4h = &ipv4h;
+    p->tcph = &tcph;
+    p->tcph->th_flags = TH_SYN|TH_RST;
 
     de = DetectFlagsParse("!PA");
 
@@ -863,17 +928,19 @@ static int FlagsTestParse09 (void) {
     sm->type = DETECT_FLAGS;
     sm->ctx = (void *)de;
 
-    ret = DetectFlagsMatch(&tv,NULL,&p,NULL,sm);
+    ret = DetectFlagsMatch(&tv,NULL,p,NULL,sm);
 
     if(ret) {
         if (de) SCFree(de);
         if (sm) SCFree(sm);
+        SCFree(p);
         return 1;
     }
 
 error:
     if (de) SCFree(de);
     if (sm) SCFree(sm);
+    SCFree(p);
     return 0;
 }
 
@@ -884,7 +951,9 @@ error:
  *  \retval 0 on failure
  */
 static int FlagsTestParse10 (void) {
-    Packet p;
+    Packet *p = SCMalloc(SIZE_OF_PACKET);
+    if (p == NULL)
+        return 0;
     ThreadVars tv;
     int ret = 0;
     DetectFlagsData *de = NULL;
@@ -893,13 +962,14 @@ static int FlagsTestParse10 (void) {
     TCPHdr tcph;
 
     memset(&tv, 0, sizeof(ThreadVars));
-    memset(&p, 0, sizeof(Packet));
+    memset(p, 0, SIZE_OF_PACKET);
+    p->pkt = (uint8_t *)(p + 1);
     memset(&ipv4h, 0, sizeof(IPV4Hdr));
     memset(&tcph, 0, sizeof(TCPHdr));
 
-    p.ip4h = &ipv4h;
-    p.tcph = &tcph;
-    p.tcph->th_flags = TH_SYN|TH_RST;
+    p->ip4h = &ipv4h;
+    p->tcph = &tcph;
+    p->tcph->th_flags = TH_SYN|TH_RST;
 
     de = DetectFlagsParse("!AP");
 
@@ -913,17 +983,19 @@ static int FlagsTestParse10 (void) {
     sm->type = DETECT_FLAGS;
     sm->ctx = (void *)de;
 
-    ret = DetectFlagsMatch(&tv,NULL,&p,NULL,sm);
+    ret = DetectFlagsMatch(&tv,NULL,p,NULL,sm);
 
     if(ret) {
         if (de) SCFree(de);
         if (sm) SCFree(sm);
+        SCFree(p);
         return 1;
     }
 
 error:
     if (de) SCFree(de);
     if (sm) SCFree(sm);
+    SCFree(p);
     return 0;
 }
 
@@ -934,7 +1006,9 @@ error:
  *  \retval 0 on failure
  */
 static int FlagsTestParse11 (void) {
-    Packet p;
+    Packet *p = SCMalloc(SIZE_OF_PACKET);
+    if (p == NULL)
+        return 0;
     ThreadVars tv;
     int ret = 0;
     DetectFlagsData *de = NULL;
@@ -943,13 +1017,14 @@ static int FlagsTestParse11 (void) {
     TCPHdr tcph;
 
     memset(&tv, 0, sizeof(ThreadVars));
-    memset(&p, 0, sizeof(Packet));
+    memset(p, 0, SIZE_OF_PACKET);
+    p->pkt = (uint8_t *)(p + 1);
     memset(&ipv4h, 0, sizeof(IPV4Hdr));
     memset(&tcph, 0, sizeof(TCPHdr));
 
-    p.ip4h = &ipv4h;
-    p.tcph = &tcph;
-    p.tcph->th_flags = TH_SYN|TH_RST|TH_URG;
+    p->ip4h = &ipv4h;
+    p->tcph = &tcph;
+    p->tcph->th_flags = TH_SYN|TH_RST|TH_URG;
 
     de = DetectFlagsParse("*AP,SR");
 
@@ -963,17 +1038,19 @@ static int FlagsTestParse11 (void) {
     sm->type = DETECT_FLAGS;
     sm->ctx = (void *)de;
 
-    ret = DetectFlagsMatch(&tv,NULL,&p,NULL,sm);
+    ret = DetectFlagsMatch(&tv,NULL,p,NULL,sm);
 
     if(ret) {
         if (de) SCFree(de);
         if (sm) SCFree(sm);
+        SCFree(p);
         return 1;
     }
 
 error:
     if (de) SCFree(de);
     if (sm) SCFree(sm);
+    SCFree(p);
     return 0;
 }
 
@@ -984,7 +1061,9 @@ error:
  *  \retval 0 on failure
  */
 static int FlagsTestParse12 (void) {
-    Packet p;
+    Packet *p = SCMalloc(SIZE_OF_PACKET);
+    if (p == NULL)
+        return 0;
     ThreadVars tv;
     int ret = 0;
     DetectFlagsData *de = NULL;
@@ -993,13 +1072,14 @@ static int FlagsTestParse12 (void) {
     TCPHdr tcph;
 
     memset(&tv, 0, sizeof(ThreadVars));
-    memset(&p, 0, sizeof(Packet));
+    memset(p, 0, SIZE_OF_PACKET);
+    p->pkt = (uint8_t *)(p + 1);
     memset(&ipv4h, 0, sizeof(IPV4Hdr));
     memset(&tcph, 0, sizeof(TCPHdr));
 
-    p.ip4h = &ipv4h;
-    p.tcph = &tcph;
-    p.tcph->th_flags = TH_SYN;
+    p->ip4h = &ipv4h;
+    p->tcph = &tcph;
+    p->tcph->th_flags = TH_SYN;
 
     de = DetectFlagsParse("0");
 
@@ -1015,17 +1095,19 @@ static int FlagsTestParse12 (void) {
     sm->type = DETECT_FLAGS;
     sm->ctx = (void *)de;
 
-    ret = DetectFlagsMatch(&tv,NULL,&p,NULL,sm);
+    ret = DetectFlagsMatch(&tv,NULL,p,NULL,sm);
 
     if(ret) {
         if (de) SCFree(de);
         if (sm) SCFree(sm);
+        SCFree(p);
         return 1;
     }
 
 error:
     if (de) SCFree(de);
     if (sm) SCFree(sm);
+    SCFree(p);
     return 0;
 }
 
@@ -1046,6 +1128,188 @@ static int FlagsTestParse13 (void) {
     return 1;
 }
 
+/**
+ * \test Parse 'C' and 'E' flags.
+ *
+ *  \retval 1 on success.
+ *  \retval 0 on failure.
+ */
+static int FlagsTestParse14(void)
+{
+    DetectFlagsData *de = DetectFlagsParse("CE");
+    if (de != NULL && (de->flags == (TH_CWR | TH_ECN)) ) {
+        DetectFlagsFree(de);
+        return 1;
+    }
+
+    return 0;
+}
+
+static int FlagsTestParse15(void)
+{
+    Packet *p = SCMalloc(SIZE_OF_PACKET);
+    if (p == NULL)
+        return 0;
+    ThreadVars tv;
+    int ret = 0;
+    DetectFlagsData *de = NULL;
+    SigMatch *sm = NULL;
+    IPV4Hdr ipv4h;
+    TCPHdr tcph;
+
+    memset(&tv, 0, sizeof(ThreadVars));
+    memset(p, 0, SIZE_OF_PACKET);
+    p->pkt = (uint8_t *)(p + 1);
+    memset(&ipv4h, 0, sizeof(IPV4Hdr));
+    memset(&tcph, 0, sizeof(TCPHdr));
+
+    p->ip4h = &ipv4h;
+    p->tcph = &tcph;
+    p->tcph->th_flags = TH_ECN | TH_CWR | TH_SYN | TH_RST;
+
+    de = DetectFlagsParse("EC+");
+
+    if (de == NULL || (de->flags != (TH_ECN | TH_CWR)) )
+        goto error;
+
+    sm = SigMatchAlloc();
+    if (sm == NULL)
+        goto error;
+
+    sm->type = DETECT_FLAGS;
+    sm->ctx = (void *)de;
+
+    ret = DetectFlagsMatch(&tv, NULL, p, NULL, sm);
+
+    if (ret) {
+        if (de)
+            SCFree(de);
+        if (sm)
+            SCFree(sm);
+        SCFree(p);
+        return 1;
+    }
+
+error:
+    if (de)
+        SCFree(de);
+    if (sm)
+        SCFree(sm);
+    SCFree(p);
+    return 0;
+}
+
+static int FlagsTestParse16(void)
+{
+    Packet *p = SCMalloc(SIZE_OF_PACKET);
+    if (p == NULL)
+        return 0;
+    ThreadVars tv;
+    int ret = 0;
+    DetectFlagsData *de = NULL;
+    SigMatch *sm = NULL;
+    IPV4Hdr ipv4h;
+    TCPHdr tcph;
+
+    memset(&tv, 0, sizeof(ThreadVars));
+    memset(p, 0, SIZE_OF_PACKET);
+    p->pkt = (uint8_t *)(p + 1);
+    memset(&ipv4h, 0, sizeof(IPV4Hdr));
+    memset(&tcph, 0, sizeof(TCPHdr));
+
+    p->ip4h = &ipv4h;
+    p->tcph = &tcph;
+    p->tcph->th_flags = TH_ECN | TH_SYN | TH_RST;
+
+    de = DetectFlagsParse("EC*");
+
+    if (de == NULL || (de->flags != (TH_ECN | TH_CWR)) )
+        goto error;
+
+    sm = SigMatchAlloc();
+    if (sm == NULL)
+        goto error;
+
+    sm->type = DETECT_FLAGS;
+    sm->ctx = (void *)de;
+
+    ret = DetectFlagsMatch(&tv, NULL, p, NULL, sm);
+
+    if (ret) {
+        if (de)
+            SCFree(de);
+        if (sm)
+            SCFree(sm);
+        SCFree(p);
+        return 1;
+    }
+
+error:
+    if (de)
+        SCFree(de);
+    if (sm)
+        SCFree(sm);
+    SCFree(p);
+    return 0;
+}
+
+/**
+ * \test Negative test.
+ */
+static int FlagsTestParse17(void)
+{
+    Packet *p = SCMalloc(SIZE_OF_PACKET);
+    if (p == NULL)
+        return 0;
+    ThreadVars tv;
+    int ret = 0;
+    DetectFlagsData *de = NULL;
+    SigMatch *sm = NULL;
+    IPV4Hdr ipv4h;
+    TCPHdr tcph;
+
+    memset(&tv, 0, sizeof(ThreadVars));
+    memset(p, 0, SIZE_OF_PACKET);
+    p->pkt = (uint8_t *)(p + 1);
+    memset(&ipv4h, 0, sizeof(IPV4Hdr));
+    memset(&tcph, 0, sizeof(TCPHdr));
+
+    p->ip4h = &ipv4h;
+    p->tcph = &tcph;
+    p->tcph->th_flags = TH_ECN | TH_SYN | TH_RST;
+
+    de = DetectFlagsParse("EC+");
+
+    if (de == NULL || (de->flags != (TH_ECN | TH_CWR)) )
+        goto error;
+
+    sm = SigMatchAlloc();
+    if (sm == NULL)
+        goto error;
+
+    sm->type = DETECT_FLAGS;
+    sm->ctx = (void *)de;
+
+    ret = DetectFlagsMatch(&tv, NULL, p, NULL, sm);
+
+    if (ret == 0) {
+        if (de)
+            SCFree(de);
+        if (sm)
+            SCFree(sm);
+        SCFree(p);
+        return 1;
+    }
+
+error:
+    if (de)
+        SCFree(de);
+    if (sm)
+        SCFree(sm);
+    SCFree(p);
+    return 0;
+}
+
 #endif /* UNITTESTS */
 
 /**
@@ -1066,5 +1330,9 @@ void FlagsRegisterTests(void) {
     UtRegisterTest("FlagsTestParse11", FlagsTestParse11, 0);
     UtRegisterTest("FlagsTestParse12", FlagsTestParse12, 0);
     UtRegisterTest("FlagsTestParse13", FlagsTestParse13, 1);
+    UtRegisterTest("FlagsTestParse14", FlagsTestParse14, 1);
+    UtRegisterTest("FlagsTestParse15", FlagsTestParse15, 1);
+    UtRegisterTest("FlagsTestParse16", FlagsTestParse16, 1);
+    UtRegisterTest("FlagsTestParse17", FlagsTestParse17, 1);
 #endif /* UNITTESTS */
 }
