@@ -104,7 +104,7 @@ int DetectIpOptsMatch (ThreadVars *t, DetectEngineThreadCtx *det_ctx, Packet *p,
     int ipopt = 0;
     DetectIpOptsData *de = (DetectIpOptsData *)m->ctx;
 
-    if(!de || !PKT_IS_IPV4(p) || !p)
+    if (!de || !PKT_IS_IPV4(p) || PKT_IS_PSEUDOPKT(p))
         return ret;
 
     /* IPV4_OPT_ANY matches on any options */
@@ -201,6 +201,8 @@ static int DetectIpOptsSetup (DetectEngineCtx *de_ctx, Signature *s, char *rawst
     sm->ctx = (void *)de;
 
     SigMatchAppendPacket(s, sm);
+    s->flags |= SIG_FLAG_REQUIRE_PACKET;
+
     return 0;
 
 error:
@@ -266,7 +268,9 @@ int IpOptsTestParse02 (void) {
  *  \retval 0 on failure
  */
 int IpOptsTestParse03 (void) {
-    Packet p;
+    Packet *p = SCMalloc(SIZE_OF_PACKET);
+    if (p == NULL)
+        return 0;
     ThreadVars tv;
     int ret = 0;
     DetectIpOptsData *de = NULL;
@@ -274,13 +278,14 @@ int IpOptsTestParse03 (void) {
     IPV4Hdr ip4h;
 
     memset(&tv, 0, sizeof(ThreadVars));
-    memset(&p, 0, sizeof(Packet));
+    memset(p, 0, SIZE_OF_PACKET);
+    p->pkt = (uint8_t *)(p + 1);
     memset(&ip4h, 0, sizeof(IPV4Hdr));
 
-    p.ip4h = &ip4h;
-    p.IPV4_OPTS[0].type = IPV4_OPT_RR;
+    p->ip4h = &ip4h;
+    p->IPV4_OPTS[0].type = IPV4_OPT_RR;
 
-    p.IPV4_OPTS_CNT++;
+    p->IPV4_OPTS_CNT++;
 
     de = DetectIpOptsParse("rr");
 
@@ -294,14 +299,17 @@ int IpOptsTestParse03 (void) {
     sm->type = DETECT_IPOPTS;
     sm->ctx = (void *)de;
 
-    ret = DetectIpOptsMatch(&tv,NULL,&p,NULL,sm);
+    ret = DetectIpOptsMatch(&tv,NULL,p,NULL,sm);
 
-    if(ret)
+    if(ret) {
+        SCFree(p);
         return 1;
+    }
 
 error:
     if (de) SCFree(de);
     if (sm) SCFree(sm);
+    SCFree(p);
     return 0;
 }
 
@@ -312,7 +320,9 @@ error:
  *  \retval 0 on failure
  */
 int IpOptsTestParse04 (void) {
-    Packet p;
+    Packet *p = SCMalloc(SIZE_OF_PACKET);
+    if (p == NULL)
+        return 0;
     ThreadVars tv;
     int ret = 0;
     DetectIpOptsData *de = NULL;
@@ -320,13 +330,14 @@ int IpOptsTestParse04 (void) {
     IPV4Hdr ip4h;
 
     memset(&tv, 0, sizeof(ThreadVars));
-    memset(&p, 0, sizeof(Packet));
+    memset(p, 0, SIZE_OF_PACKET);
+    p->pkt = (uint8_t *)(p + 1);
     memset(&ip4h, 0, sizeof(IPV4Hdr));
 
-    p.ip4h = &ip4h;
-    p.IPV4_OPTS[0].type = IPV4_OPT_RR;
+    p->ip4h = &ip4h;
+    p->IPV4_OPTS[0].type = IPV4_OPT_RR;
 
-    p.IPV4_OPTS_CNT++;
+    p->IPV4_OPTS_CNT++;
 
     de = DetectIpOptsParse("lsrr");
 
@@ -340,14 +351,17 @@ int IpOptsTestParse04 (void) {
     sm->type = DETECT_IPOPTS;
     sm->ctx = (void *)de;
 
-    ret = DetectIpOptsMatch(&tv,NULL,&p,NULL,sm);
+    ret = DetectIpOptsMatch(&tv,NULL,p,NULL,sm);
 
-    if(ret)
+    if(ret) {
+        SCFree(p);
         return 1;
+    }
 
 error:
     if (de) SCFree(de);
     if (sm) SCFree(sm);
+    SCFree(p);
     return 0;
 }
 #endif /* UNITTESTS */
