@@ -24,25 +24,27 @@
 #ifndef __DETECT_PCRE_H__
 #define __DETECT_PCRE_H__
 
-#define DETECT_PCRE_RELATIVE        0x0001
-#define DETECT_PCRE_RAWBYTES        0x0002
-#define DETECT_PCRE_URI             0x0004
+#define DETECT_PCRE_RELATIVE            0x0001
+#define DETECT_PCRE_RAWBYTES            0x0002
+#define DETECT_PCRE_URI                 0x0004
 
-#define DETECT_PCRE_CAPTURE_PKT     0x0008
-#define DETECT_PCRE_CAPTURE_FLOW    0x0010
-#define DETECT_PCRE_MATCH_LIMIT     0x0020
+#define DETECT_PCRE_CAPTURE_PKT         0x0008
+#define DETECT_PCRE_CAPTURE_FLOW        0x0010
+#define DETECT_PCRE_MATCH_LIMIT         0x0020
 
-#define DETECT_PCRE_HTTP_BODY_AL    0x0040
-#define DETECT_PCRE_RELATIVE_NEXT   0x0080
+#define DETECT_PCRE_HTTP_CLIENT_BODY    0x0040
+#define DETECT_PCRE_HTTP_SERVER_BODY    0x0080
+
+#define DETECT_PCRE_RELATIVE_NEXT       0x0100
 
 /* new modifiers 2.8.5.3 support */
-#define DETECT_PCRE_HEADER          0x0100
-#define DETECT_PCRE_RAW_HEADER      0x0200
-#define DETECT_PCRE_COOKIE          0x0400
-#define DETECT_PCRE_METHOD          0x0800
-#define DETECT_PCRE_HTTP_RAW_URI    0x1000
+#define DETECT_PCRE_HEADER              0x0200
+#define DETECT_PCRE_RAW_HEADER          0x0400
+#define DETECT_PCRE_COOKIE              0x0800
+#define DETECT_PCRE_METHOD              0x1000
+#define DETECT_PCRE_HTTP_RAW_URI        0x2000
 
-#define DETECT_PCRE_NEGATE          0x2000
+#define DETECT_PCRE_NEGATE              0x4000
 
 typedef struct DetectPcreData_ {
     /* pcre options */
@@ -57,7 +59,6 @@ typedef struct DetectPcreData_ {
 /* prototypes */
 int DetectPcrePayloadMatch(DetectEngineThreadCtx *, Signature *, SigMatch *, Packet *, Flow *, uint8_t *, uint32_t);
 int DetectPcrePacketPayloadMatch(DetectEngineThreadCtx *, Packet *, Signature *, SigMatch *);
-//int DetectPcrePayloadMatch(DetectEngineThreadCtx *, Packet *, Signature *, SigMatch *);
 int DetectPcrePayloadDoMatch(DetectEngineThreadCtx *, Signature *, SigMatch *,
                              Packet *, uint8_t *, uint16_t);
 void DetectPcreRegister (void);
