@@ -18,7 +18,7 @@
 /**
  * \file
  *
- * \author Anoop Saldanha <poonaatsoc@gmail.com>
+ * \author Anoop Saldanha <anoopsaldanha@gmail.com>
  *
  * Implements support for ssl_state keyword.
  */
@@ -716,7 +716,6 @@ static int DetectSslStateTest07(void)
     f.alproto = ALPROTO_TLS;
 
     StreamTcpInitConfig(TRUE);
-    FlowL7DataPtrInit(&f);
 
     de_ctx = DetectEngineCtxInit();
     if (de_ctx == NULL)
@@ -754,14 +753,14 @@ static int DetectSslStateTest07(void)
     SigGroupBuild(de_ctx);
     DetectEngineThreadCtxInit(&th_v, (void *)de_ctx, (void *)&det_ctx);
 
-    r = AppLayerParse(&f, ALPROTO_TLS, STREAM_TOSERVER | STREAM_START, chello_buf,
+    r = AppLayerParse(NULL, &f, ALPROTO_TLS, STREAM_TOSERVER | STREAM_START, chello_buf,
                       chello_buf_len);
     if (r != 0) {
         printf("toserver chunk 1 returned %" PRId32 ", expected 0: ", r);
         goto end;
     }
 
-    ssl_state = f.aldata[AlpGetStateIdx(ALPROTO_TLS)];
+    ssl_state = f.alstate;
     if (ssl_state == NULL) {
         printf("no ssl state: ");
         goto end;
@@ -779,7 +778,7 @@ static int DetectSslStateTest07(void)
     if (PacketAlertCheck(p, 4))
         goto end;
 
-    r = AppLayerParse(&f, ALPROTO_TLS, STREAM_TOCLIENT, shello_buf,
+    r = AppLayerParse(NULL, &f, ALPROTO_TLS, STREAM_TOCLIENT, shello_buf,
                       shello_buf_len);
     if (r != 0) {
         printf("toclient chunk 1 returned %" PRId32 ", expected 0: ", r);
@@ -798,7 +797,7 @@ static int DetectSslStateTest07(void)
     if (PacketAlertCheck(p, 4))
         goto end;
 
-    r = AppLayerParse(&f, ALPROTO_TLS, STREAM_TOSERVER, client_change_cipher_spec_buf,
+    r = AppLayerParse(NULL, &f, ALPROTO_TLS, STREAM_TOSERVER, client_change_cipher_spec_buf,
                       client_change_cipher_spec_buf_len);
     if (r != 0) {
         printf("toserver chunk 1 returned %" PRId32 ", expected 0: ", r);
@@ -818,7 +817,7 @@ static int DetectSslStateTest07(void)
     if (PacketAlertCheck(p, 4))
         goto end;
 
-    r = AppLayerParse(&f, ALPROTO_TLS, STREAM_TOCLIENT, server_change_cipher_spec_buf,
+    r = AppLayerParse(NULL, &f, ALPROTO_TLS, STREAM_TOCLIENT, server_change_cipher_spec_buf,
                       server_change_cipher_spec_buf_len);
     if (r != 0) {
         printf("toclient chunk 1 returned %" PRId32 ", expected 0: ", r);
@@ -838,7 +837,7 @@ static int DetectSslStateTest07(void)
     if (PacketAlertCheck(p, 4))
         goto end;
 
-    r = AppLayerParse(&f, ALPROTO_TLS, STREAM_TOSERVER, toserver_app_data_buf,
+    r = AppLayerParse(NULL, &f, ALPROTO_TLS, STREAM_TOSERVER, toserver_app_data_buf,
                       toserver_app_data_buf_len);
     if (r != 0) {
         printf("toserver chunk 1 returned %" PRId32 ", expected 0: ", r);
@@ -867,7 +866,6 @@ static int DetectSslStateTest07(void)
     DetectEngineThreadCtxDeinit(&th_v, (void *)det_ctx);
     DetectEngineCtxFree(de_ctx);
 
-    FlowL7DataPtrFree(&f);
     StreamTcpFreeConfig(TRUE);
     FLOW_DESTROY(&f);
     UTHFreePackets(&p, 1);
