@@ -88,6 +88,8 @@ enum {
     DETECT_SM_LIST_TMATCH,
     /* list for http_client_body keyword and the ones relative to it */
     DETECT_SM_LIST_HCBDMATCH,
+    /* list for http_server_body keyword and the ones relative to it */
+    DETECT_SM_LIST_HSBDMATCH,
     /* list for http_header keyword and the ones relative to it */
     DETECT_SM_LIST_HHDMATCH,
     /* list for http_raw_header keyword and the ones relative to it */
@@ -98,6 +100,10 @@ enum {
     DETECT_SM_LIST_HCDMATCH,
     /* list for http_raw_uri keyword and the ones relative to it */
     DETECT_SM_LIST_HRUDMATCH,
+
+    DETECT_SM_LIST_FILEMATCH,
+
+    DETECT_SM_LIST_POSTMATCH,
     DETECT_SM_LIST_MAX,
 };
 
@@ -216,58 +222,44 @@ typedef struct DetectPort_ {
 } DetectPort;
 
 /* Signature flags */
-#define SIG_FLAG_RECURSIVE      (1)  /**< recursive capturing enabled */
-#define SIG_FLAG_SRC_ANY        (1<<1) /**< source is any */
-#define SIG_FLAG_DST_ANY        (1<<2)  /**< destination is any */
-#define SIG_FLAG_SP_ANY         (1<<3) /**< source port is any */
+#define SIG_FLAG_RECURSIVE              (1)     /**< recursive capturing enabled */
 
-#define SIG_FLAG_DP_ANY         (1<<4)    /**< destination port is any */
-#define SIG_FLAG_NOALERT        (1<<5)  /**< no alert flag is set */
+#define SIG_FLAG_SRC_ANY                (1<<1)  /**< source is any */
+#define SIG_FLAG_DST_ANY                (1<<2)  /**< destination is any */
+#define SIG_FLAG_SP_ANY                 (1<<3)  /**< source port is any */
+#define SIG_FLAG_DP_ANY                 (1<<4)  /**< destination port is any */
 
-#define SIG_FLAG_MPM            (1<<6)  /**< sig has mpm portion (content) */
-#define SIG_FLAG_MPM_URI        (1<<7) /**< sig has mpm portion (uricontent) */
-#define SIG_FLAG_DSIZE          (1<<8)  /**< signature has a dsize setting */
+#define SIG_FLAG_NOALERT                (1<<5)  /**< no alert flag is set */
+#define SIG_FLAG_DSIZE                  (1<<6)  /**< signature has a dsize setting */
+#define SIG_FLAG_APPLAYER               (1<<7)  /**< signature applies to app layer instead of packets */
+#define SIG_FLAG_IPONLY                 (1<<8) /**< ip only signature */
 
-#define SIG_FLAG_APPLAYER       (1<<9)   /**< signature applies to app layer instead of packets */
-#define SIG_FLAG_IPONLY         (1<<10) /**< ip only signature */
+#define SIG_FLAG_STATE_MATCH            (1<<9) /**< signature has matches that require stateful inspection */
 
-#define SIG_FLAG_STATE_MATCH                    (1<<11)  /**< signature has matches that require stateful inspection */
-#define SIG_FLAG_REQUIRE_PACKET  		(1<<12)  /**< signature is requiring packet match */
-#define SIG_FLAG_MPM_PACKET                     (1<<13)
-#define SIG_FLAG_MPM_PACKET_NEG                 (1<<14)
+#define SIG_FLAG_REQUIRE_PACKET         (1<<10) /**< signature is requiring packet match */
+#define SIG_FLAG_REQUIRE_STREAM         (1<<11) /**< signature is requiring stream match */
 
-#define SIG_FLAG_MPM_STREAM                     (1<<15)
-#define SIG_FLAG_MPM_STREAM_NEG                 (1<<16)
+#define SIG_FLAG_MPM_PACKET             (1<<12)
+#define SIG_FLAG_MPM_PACKET_NEG         (1<<13)
+#define SIG_FLAG_MPM_STREAM             (1<<14)
+#define SIG_FLAG_MPM_STREAM_NEG         (1<<15)
+#define SIG_FLAG_MPM_HTTP               (1<<16)
+#define SIG_FLAG_MPM_HTTP_NEG           (1<<17)
 
-#define SIG_FLAG_MPM_URICONTENT                 (1<<17)
-#define SIG_FLAG_MPM_URICONTENT_NEG             (1<<18)
+#define SIG_FLAG_REQUIRE_FLOWVAR        (1<<18) /**< signature can only match if a flowbit, flowvar or flowint is available. */
 
-#define SIG_FLAG_MPM_HHDCONTENT                 (1<<19)
-#define SIG_FLAG_MPM_HHDCONTENT_NEG             (1<<20)
+#define SIG_FLAG_FILESTORE              (1<<19) /**< signature has filestore keyword */
 
-#define SIG_FLAG_MPM_HRHDCONTENT                (1<<21)
-#define SIG_FLAG_MPM_HRHDCONTENT_NEG            (1<<22)
-
-#define SIG_FLAG_MPM_HCBDCONTENT                (1<<23)
-#define SIG_FLAG_MPM_HCBDCONTENT_NEG            (1<<24)
-
-#define SIG_FLAG_MPM_HMDCONTENT                 (1<<25)
-#define SIG_FLAG_MPM_HMDCONTENT_NEG             (1<<26)
-
-#define SIG_FLAG_MPM_HCDCONTENT                 (1<<27)
-#define SIG_FLAG_MPM_HCDCONTENT_NEG             (1<<28)
-
-#define SIG_FLAG_MPM_HRUDCONTENT                (1<<29)
-#define SIG_FLAG_MPM_HRUDCONTENT_NEG            (1<<30)
-
-#define SIG_FLAG_REQUIRE_FLOWVAR                (1<<31) /**< signature can only match if a flowbit, flowvar or flowint is available. */
+#define SIG_FLAG_TOSERVER               (1<<20)
+#define SIG_FLAG_TOCLIENT               (1<<21)
 
 /* signature init flags */
-#define SIG_FLAG_DEONLY         1  /**< decode event only signature */
-#define SIG_FLAG_PACKET         (1<<1)  /**< signature has matches against a packet (as opposed to app layer) */
-#define SIG_FLAG_FLOW           (1<<2)  /**< signature has a flow setting */
-#define SIG_FLAG_BIDIREC        (1<<3)  /**< signature has bidirectional operator */
-#define SIG_FLAG_PAYLOAD        (1<<4)  /**< signature is inspecting the packet payload */
+#define SIG_FLAG_INIT_DEONLY         1  /**< decode event only signature */
+#define SIG_FLAG_INIT_PACKET         (1<<1)  /**< signature has matches against a packet (as opposed to app layer) */
+#define SIG_FLAG_INIT_FLOW           (1<<2)  /**< signature has a flow setting */
+#define SIG_FLAG_INIT_BIDIREC        (1<<3)  /**< signature has bidirectional operator */
+#define SIG_FLAG_INIT_PAYLOAD        (1<<4)  /**< signature is inspecting the packet payload */
+#define SIG_FLAG_INIT_FILE_DATA      (1<<5)  /**< file_data set */
 
 /* signature mask flags */
 #define SIG_MASK_REQUIRE_PAYLOAD            1
@@ -285,6 +277,11 @@ typedef struct DetectPort_ {
 #define DETECT_ENGINE_THREAD_CTX_INSPECTING_PACKET 0x0001
 #define DETECT_ENGINE_THREAD_CTX_INSPECTING_STREAM 0x0002
 
+#define FILE_SIG_NEED_FILE          0x01
+#define FILE_SIG_NEED_FILENAME      0x02
+#define FILE_SIG_NEED_TYPE          0x04
+#define FILE_SIG_NEED_MAGIC         0x08    /**< need the start of the file */
+#define FILE_SIG_NEED_FILECONTENT   0x10
 
 /* Detection Engine flags */
 #define DE_QUIET           0x01     /**< DE is quiet (esp for unittests) */
@@ -320,18 +317,19 @@ typedef struct SignatureHeader_ {
     union {
         struct {
             uint16_t alproto;
-            uint16_t mpm_stream_pattern_id_div_8;
-            uint8_t mpm_stream_pattern_id_mod_8;
             SigIntId num; /**< signature number, internal id */
         };
-        uint64_t hdr_copy2;
+        uint32_t hdr_copy2;
     };
     union {
         struct {
-            /** pattern in the mpm matcher */
-            PatIntId mpm_http_pattern_id;
+            SigIntId order_id;
+
+            /** inline -- action */
+            uint8_t action;
+            uint8_t file_flags;
         };
-        uint64_t hdr_copy3;
+        uint32_t hdr_copy3;
     };
 
     /** pointer to the full signature */
@@ -361,38 +359,21 @@ typedef struct Signature_ {
     union {
         struct {
             uint16_t alproto;
-            uint16_t mpm_stream_pattern_id_div_8;
-            uint8_t mpm_stream_pattern_id_mod_8;
             SigIntId num; /**< signature number, internal id */
         };
-        uint64_t hdr_copy2;
+        uint32_t hdr_copy2;
     };
     union {
         struct {
-            PatIntId mpm_http_pattern_id;
+            SigIntId order_id;
+
+            /** inline -- action */
+            uint8_t action;
+            uint8_t file_flags;
         };
-        uint64_t hdr_copy3;
+        uint32_t hdr_copy3;
     };
 
-    /* mpm flags */
-//    uint32_t mpm_flags;
-
-    //PatIntId mpm_pattern_id;
-    //PatIntId mpm_stream_pattern_id;
-
-/*
-    //PatIntId mpm_pattern_id;
-    //PatIntId mpm_stream_pattern_id;
-    uint16_t mpm_pattern_id_div_8;
-    uint8_t mpm_pattern_id_mod_8;
-    uint8_t pad0;
-    //PatIntId mpm_pattern_id;
-    //PatIntId mpm_stream_pattern_id;
-    uint16_t mpm_stream_pattern_id_div_8;
-    uint8_t mpm_stream_pattern_id_mod_8;
-    uint8_t pad1;
-*/
-
     /* the fast pattern added from this signature */
     SigMatch *mpm_sm;
 
@@ -420,15 +401,9 @@ typedef struct Signature_ {
     uint16_t mpm_content_maxlen;
     uint16_t mpm_uricontent_maxlen;
 
-
     /** number of sigmatches in the match and pmatch list */
     uint16_t sm_cnt;
 
-    SigIntId order_id;
-
-    /** inline -- action */
-    uint8_t action;
-
     uint32_t id;  /**< sid, set by the 'sid' rule keyword */
     uint32_t gid; /**< generator id */
     uint32_t rev;
@@ -436,9 +411,6 @@ typedef struct Signature_ {
     /** classification id **/
     uint8_t class;
 
-    /* signature match mask */
-    //SignatureMask mask;
-
     int prio;
 
     char *msg;
@@ -654,6 +626,8 @@ typedef struct DetectEngineCtx_ {
 
     /* conf parameter that limits the length of the http request body inspected */
     int hcbd_buffer_limit;
+    /* conf parameter that limits the length of the http response body inspected */
+    int hsbd_buffer_limit;
 
     /* array containing all sgh's in use so we can loop
      * through it in Stage4. */
@@ -661,10 +635,13 @@ typedef struct DetectEngineCtx_ {
     uint32_t sgh_array_cnt;
     uint32_t sgh_array_size;
 
-    int32_t sgh_mpm_context_packet;
+    int32_t sgh_mpm_context_proto_tcp_packet;
+    int32_t sgh_mpm_context_proto_udp_packet;
+    int32_t sgh_mpm_context_proto_other_packet;
     int32_t sgh_mpm_context_stream;
     int32_t sgh_mpm_context_uri;
     int32_t sgh_mpm_context_hcbd;
+    int32_t sgh_mpm_context_hsbd;
     int32_t sgh_mpm_context_hhd;
     int32_t sgh_mpm_context_hrhd;
     int32_t sgh_mpm_context_hmd;
@@ -694,6 +671,8 @@ enum {
     ENGINE_SGH_MPM_FACTORY_CONTEXT_AUTO
 };
 
+#define DETECT_FILESTORE_MAX 15
+
 /**
   * Detection engine thread data.
   */
@@ -709,29 +688,20 @@ typedef struct DetectionEngineThreadCtx_ {
     /* used by pcre match function alone */
     uint32_t pcre_match_start_offset;
 
-    /* http_uri stuff for uricontent */
-    //char de_have_httpuri;
-    //char de_mpm_scanned_uri;
-
-    /* detectione engine context for hcbd mpm */
-    //char de_have_hcbd;
-    //char de_mpm_scanned_hcbd;
-
-    /* detectione engine context for hhd mpm */
-    //char de_have_hhd;
-    //char de_mpm_scanned_hhd;
-
-    /* detectione engine context for hrhd mpm */
-    //char de_have_hrhd;
-    //char de_mpm_scanned_hrhd;
-
     uint8_t **hcbd_buffers;
     uint32_t *hcbd_buffers_len;
     uint16_t hcbd_buffers_list_len;
 
+    /* counter for the filestore array below -- up here for cache reasons. */
+    uint16_t filestore_cnt;
+
+    uint16_t hhd_buffers_list_len;
+    uint16_t hsbd_buffers_list_len;
+    uint8_t **hsbd_buffers;
+    uint32_t *hsbd_buffers_len;
+
     uint8_t **hhd_buffers;
     uint32_t *hhd_buffers_len;
-    uint16_t hhd_buffers_list_len;
 
     /** id for alert counter */
     uint16_t counter_alerts;
@@ -774,6 +744,9 @@ typedef struct DetectionEngineThreadCtx_ {
     PatternMatcherQueue pmq;
     PatternMatcherQueue smsg_pmq[256];
 
+    /** ID of the transaction currently being inspected. */
+    uint16_t tx_id;
+
     /* counters */
     uint32_t pkts;
     uint32_t pkts_searched;
@@ -798,6 +771,15 @@ typedef struct DetectionEngineThreadCtx_ {
     /* string to replace */
     DetectReplaceList *replist;
 
+    /* Array in which the filestore keyword stores file id and tx id. If the
+     * full signature matches, these are processed by a post-match filestore
+     * function to finalize the store. */
+    struct {
+        uint16_t file_id;
+        uint16_t tx_id;
+    } filestore[DETECT_FILESTORE_MAX];
+    SigMatch *filestore_sm;
+
     DetectEngineCtx *de_ctx;
 #ifdef __SC_CUDA_SUPPORT__
     /* each detection thread would have it's own queue where the cuda dispatcher
@@ -849,6 +831,9 @@ typedef struct SigTableElmt_ {
 #define SIG_GROUP_HEAD_MPM_HCD          0x00100000
 #define SIG_GROUP_HEAD_MPM_HRUD         0x00200000
 #define SIG_GROUP_HEAD_REFERENCED       0x00400000 /**< sgh is being referenced by others, don't clear */
+#define SIG_GROUP_HEAD_HAVEFILEMAGIC    0x00800000
+#define SIG_GROUP_HAVEHSBDCONTENT       0x01000000
+#define SIG_GROUP_HEAD_MPM_HSBD         0x02000000
 
 typedef struct SigGroupHeadInitData_ {
     /* list of content containers
@@ -891,21 +876,24 @@ typedef struct SigGroupHead_ {
     SignatureHeader *head_array;
 
     /* pattern matcher instances */
-    MpmCtx *mpm_ctx;
+    MpmCtx *mpm_proto_other_ctx;
+    MpmCtx *mpm_proto_tcp_ctx;
+    MpmCtx *mpm_proto_udp_ctx;
     MpmCtx *mpm_stream_ctx;
     MpmCtx *mpm_uri_ctx;
     MpmCtx *mpm_hcbd_ctx;
+    MpmCtx *mpm_hsbd_ctx;
     MpmCtx *mpm_hhd_ctx;
     MpmCtx *mpm_hrhd_ctx;
     MpmCtx *mpm_hmd_ctx;
     MpmCtx *mpm_hcd_ctx;
     MpmCtx *mpm_hrud_ctx;
 
-    uint16_t mpm_streamcontent_maxlen;
     uint16_t mpm_uricontent_maxlen;
-#if __WORDSIZE == 64
-    uint32_t pad2;
-#endif
+
+    /** the number of signatures in this sgh that have the filestore keyword
+     *  set. */
+    uint16_t filestore_cnt;
 
     /** Array with sig ptrs... size is sig_cnt * sizeof(Signature *) */
     Signature **match_array;
@@ -981,6 +969,7 @@ enum {
     DETECT_TTL,
     DETECT_ITYPE,
     DETECT_ICODE,
+    DETECT_TOS,
     DETECT_ICMP_ID,
     DETECT_ICMP_SEQ,
     DETECT_DETECTION_FILTER,
@@ -1001,6 +990,7 @@ enum {
     DETECT_AL_HTTP_METHOD,
     DETECT_AL_URILEN,
     DETECT_AL_HTTP_CLIENT_BODY,
+    DETECT_AL_HTTP_SERVER_BODY,
     DETECT_AL_HTTP_HEADER,
     DETECT_AL_HTTP_RAW_HEADER,
     DETECT_AL_HTTP_URI,
@@ -1012,6 +1002,8 @@ enum {
     DETECT_AL_SSL_VERSION,
     DETECT_AL_SSL_STATE,
     DETECT_BYTE_EXTRACT,
+    DETECT_FILE_DATA,
+    DETECT_AL_APP_LAYER_EVENT,
 
     DETECT_DCE_IFACE,
     DETECT_DCE_OPNUM,
@@ -1022,6 +1014,11 @@ enum {
     DETECT_ENGINE_EVENT,
     DETECT_STREAM_EVENT,
 
+    DETECT_FILENAME,
+    DETECT_FILEEXT,
+    DETECT_FILESTORE,
+    DETECT_FILEMAGIC,
+
     /* make sure this stays last */
     DETECT_TBLSIZE,
 };
@@ -1043,7 +1040,7 @@ int SigGroupBuild(DetectEngineCtx *);
 int SigGroupCleanup (DetectEngineCtx *de_ctx);
 void SigAddressPrepareBidirectionals (DetectEngineCtx *);
 
-int SigLoadSignatures (DetectEngineCtx *, char *);
+int SigLoadSignatures (DetectEngineCtx *, char *, int);
 void SigTableSetup(void);
 int SigMatchSignatures(ThreadVars *th_v, DetectEngineCtx *de_ctx,
                        DetectEngineThreadCtx *det_ctx, Packet *p);
@@ -1053,5 +1050,8 @@ SigGroupHead *SigMatchSignaturesGetSgh(DetectEngineCtx *de_ctx, DetectEngineThre
 
 Signature *DetectGetTagSignature(void);
 
+int SignatureIsFilestoring(Signature *);
+int SignatureIsFilemagicInspecting(Signature *);
+
 #endif /* __DETECT_H__ */
 
