@@ -18,12 +18,17 @@
 /**
  * \file
  *
- * \author Anoop Saldanha <poonaatsoc@gmail.com>
+ * \author Anoop Saldanha <anoopsaldanha@gmail.com>
  */
 
 #ifndef __FLOW_MANAGER_H__
 #define __FLOW_MANAGER_H__
 
+SCCondT flow_manager_cond;
+SCMutex flow_manager_mutex;
+
+#define FlowWakeupFlowManagerThread() SCCondSignal(&flow_manager_cond)
+
 void FlowManagerThreadSpawn(void);
 void FlowKillFlowManagerThread(void);
 
