@@ -108,7 +108,7 @@ void *ParseAFPConfig(const char *iface)
     char *tmpclusterid;
     char *tmpctype;
     intmax_t value;
-    int dispromisc;
+    int boolval;
 
     if (iface == NULL) {
         return NULL;
@@ -125,6 +125,7 @@ void *ParseAFPConfig(const char *iface)
     aconf->cluster_id = 1;
     aconf->cluster_type = PACKET_FANOUT_HASH;
     aconf->promisc = 1;
+    aconf->checksum_mode = CHECKSUM_VALIDATION_KERNEL;
     aconf->DerefFunc = AFPDerefConfig;
 
     /* Find initial node */
@@ -164,7 +165,7 @@ void *ParseAFPConfig(const char *iface)
     }
 
     if (ConfGetChildValue(if_root, "cluster-type", &tmpctype) != 1) {
-        SCLogError(SC_ERR_GET_CLUSTER_TYPE_FAILED,"Could not get cluster-type fron config");
+        SCLogError(SC_ERR_GET_CLUSTER_TYPE_FAILED,"Could not get cluster-type from config");
     } else if (strcmp(tmpctype, "cluster_round_robin") == 0) {
         SCLogInfo("Using round-robin cluster mode for AF_PACKET (iface %s)",
                 aconf->iface);
@@ -197,13 +198,27 @@ void *ParseAFPConfig(const char *iface)
         aconf->buffer_size = 0;
     }
 
-    ConfGetChildValueBool(if_root, "disable-promisc", (int *)&dispromisc);
-    if (dispromisc) {
+    ConfGetChildValueBool(if_root, "disable-promisc", (int *)&boolval);
+    if (boolval) {
         SCLogInfo("Disabling promiscuous mode on iface %s",
                 aconf->iface);
         aconf->promisc = 0;
     }
 
+    if (ConfGetChildValue(if_root, "checksum-checks", &tmpctype) == 1) {
+        if (strcmp(tmpctype, "auto") == 0) {
+            aconf->checksum_mode = CHECKSUM_VALIDATION_AUTO;
+        } else if (strcmp(tmpctype, "yes") == 0) {
+            aconf->checksum_mode = CHECKSUM_VALIDATION_ENABLE;
+        } else if (strcmp(tmpctype, "no") == 0) {
+            aconf->checksum_mode = CHECKSUM_VALIDATION_DISABLE;
+        } else if (strcmp(tmpctype, "kernel") == 0) {
+            aconf->checksum_mode = CHECKSUM_VALIDATION_KERNEL;
+        } else {
+            SCLogError(SC_ERR_INVALID_ARGUMENT, "Invalid value for checksum-checks for %s", aconf->iface);
+        }
+    }
+
     return aconf;
 }
 
@@ -246,7 +261,9 @@ int RunModeIdsAFPAuto(DetectEngineCtx *de_ctx)
     ConfGet("af-packet.live-interface", &live_dev);
 
     ret = RunModeSetLiveCaptureAuto(de_ctx,
-                                    ParseAFPConfig, "ReceiveAFP",
+                                    ParseAFPConfig,
+                                    AFPConfigGeThreadsCount,
+                                    "ReceiveAFP",
                                     "DecodeAFP", "RecvAFP",
                                     live_dev);
     if (ret != 0) {
