@@ -43,6 +43,10 @@ typedef struct TmModule_ {
 
     TmEcode (*PktAcqLoop)(ThreadVars *, void *, void *);
 
+    /** global Init/DeInit */
+    TmEcode (*Init)(void);
+    TmEcode (*DeInit)(void);
+
     void (*RegisterTests)(void);
 
     uint8_t cap_flags;   /**< Flags to indicate the capability requierment of
@@ -105,5 +109,8 @@ void TmModuleDebugList(void);
 void TmModuleRegisterTests(void);
 const char * TmModuleTmmIdToString(TmmId id);
 
+void TmModuleRunInit(void);
+void TmModuleRunDeInit(void);
+
 #endif /* __TM_MODULES_H__ */
 
