@@ -1,4 +1,4 @@
-/* Copyright (C) 2011 Open Information Security Foundation
+/* Copyright (C) 2011-2012 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -25,6 +25,8 @@
 
 #include "suricata-common.h"
 
+#include "util-checksum.h"
+
 int ReCalculateChecksum(Packet *p)
 {
     if (PKT_IS_IPV4(p)) {
@@ -57,3 +59,32 @@ int ReCalculateChecksum(Packet *p)
 
     return 0;
 }
+
+/**
+ *  \brief Check if the number of invalid checksums indicate checksum
+ *         offloading in place.
+ *
+ *  \retval 1 yes, offloading in place
+ *  \retval 0 no, no offloading used
+ */
+int ChecksumAutoModeCheck(uint32_t thread_count,
+        unsigned int iface_count, unsigned int iface_fail)
+{
+    if (thread_count == CHECKSUM_SAMPLE_COUNT) {
+        if (iface_fail != 0) {
+            if ((iface_count / iface_fail) < CHECKSUM_INVALID_RATIO) {
+                SCLogInfo("More than 1/%dth of packets have an invalid "
+                        "checksum, assuming checksum offloading is used (%d/%d)",
+                        CHECKSUM_INVALID_RATIO, iface_fail, iface_count);
+                return 1;
+            } else {
+                SCLogInfo("Less than 1/%dth of packets have an invalid "
+                        "checksum, assuming checksum offloading is NOT used (%d/%d)",
+                        CHECKSUM_INVALID_RATIO, iface_fail, iface_count);
+            }
+        } else {
+            SCLogInfo("No packets with invalid checksum, assuming checksum offloading is NOT used");
+        }
+    }
+    return 0;
+}
