@@ -21,12 +21,16 @@
  * \author Victor Julien <victor@inliniac.net>
  */
 
+
+
 #ifndef __UTIL_PRINT_H__
 #define __UTIL_PRINT_H__
 
 void PrintRawLineHexFp(FILE *, uint8_t *, uint32_t);
 void PrintRawUriFp(FILE *, uint8_t *, uint32_t);
 void PrintRawDataFp(FILE *, uint8_t *, uint32_t);
+void PrintRawLineHexBuf(char *, uint32_t, uint8_t *, uint32_t );
+const char *PrintInet(int , const void *, char *, socklen_t);
 
 #endif /* __UTIL_PRINT_H__ */
 
