@@ -121,6 +121,8 @@ void VariableNameFreeHash() {
     if (variable_names != NULL) {
         HashListTableFree(variable_names);
         HashListTableFree(variable_idxs);
+        variable_names = NULL;
+        variable_idxs = NULL;
     }
 }
 
@@ -132,7 +134,6 @@ void VariableNameFreeHash() {
  */
 uint16_t VariableNameGetIdx(char *name, uint8_t type) {
     uint16_t idx = 0;
-    VariableName *lookup_fn = NULL;
 
     VariableName *fn = SCMalloc(sizeof(VariableName));
     if (fn == NULL)
@@ -145,7 +146,7 @@ uint16_t VariableNameGetIdx(char *name, uint8_t type) {
     if (fn->name == NULL)
         goto error;
 
-    lookup_fn = (VariableName *)HashListTableLookup(variable_names, (void *)fn, 0);
+    VariableName *lookup_fn = (VariableName *)HashListTableLookup(variable_names, (void *)fn, 0);
     if (lookup_fn == NULL) {
         variable_names_idx++;
 
@@ -171,8 +172,6 @@ error:
  */
 char *VariableIdxGetName(uint16_t idx, uint8_t type)
 {
-    VariableName *lookup_fn = NULL;
-
     VariableName *fn = SCMalloc(sizeof(VariableName));
     if (fn == NULL)
         goto error;
@@ -183,7 +182,7 @@ char *VariableIdxGetName(uint16_t idx, uint8_t type)
     fn->type = type;
     fn->idx = idx;
 
-    lookup_fn = (VariableName *)HashListTableLookup(variable_idxs, (void *)fn, 0);
+    VariableName *lookup_fn = (VariableName *)HashListTableLookup(variable_idxs, (void *)fn, 0);
     if (lookup_fn != NULL) {
         name = SCStrdup(lookup_fn->name);
         if (name == NULL)
